import os

from twisted.web.server import Site
from twisted.web.resource import Resource
from twisted.application import internet, service
from twisted.web.static import File

from jinja2 import Environment, PackageLoader

try:
    import json
except:
        import simplejson as json


class WebMessageDisplay(Resource):

    def __init__(self, name, storage):
        Resource.__init__(self)
        self.name = name
        self.storage = storage

    def render_GET(self, request):
        email_list = self.storage.get_for_name(self.name)
        decoded_email_list = []
        available_content_types = set()
        if email_list:
            for msg, attachments in email_list:
                parts = {}
                parts['headers'] = msg.items()
                parts['content_type'] = msg.get_content_type()
                for part in msg.walk():
                    # multipart/* are just containers, and things that
                    # don't start with 'text/' are assumed to be
                    # attachments
                    is_multipart = (part.get_content_maintype() == 'multipart')
                    is_text = part.get_content_type().startswith('text/')
                    if is_multipart or not is_text:
                        continue

                    content_type = part.get_content_type()
                    # Assume that any non-text types are just attachments
                    if content_type.startswith('text/'):
                        payload = part.get_payload(decode=True)
                        if isinstance(payload, unicode):
                            output = payload
                        else:
                            output = unicode(payload, 'utf-8')

                    parts[part.get_content_subtype()] = output
                    available_content_types.add(part.get_content_subtype())

                # Output the names of the attachments.
                for i, attachment_path in enumerate(attachments, start=1):
                    heading = 'attachment {}'.format(i)
                    parts[heading] = os.path.basename(attachment_path)
                    available_content_types.add(heading)

                decoded_email_list.append(parts)

        data = {'email_list': decoded_email_list,
                'address_name': self.name,
                'content_types': list(available_content_types)}

        env = Environment(loader=PackageLoader('fakeemail', 'templates'))
        template = env.get_template('message_display.html')

        return template.render(data).encode('UTF-8')


class WebMessageRootDisplay(Resource):

    def __init__(self, name, storage):
        Resource.__init__(self)
        self.storage = storage

    def internal_render(self, user_message=None):
        address_list = [[self.storage.get_count(name), name]
                        for name in self.storage.get_all_names()]
        data = {'address_list': address_list, 'user_message': user_message}

        env = Environment(loader=PackageLoader('fakeemail', 'templates'))
        template = env.get_template('home_page.html')

        return template.render(data).encode('utf-8')

    def render_GET(self, request, user_message=None):

        return self.internal_render()

    def render_POST(self, request):
        self.storage.clear_all()
        return self.internal_render('Data Cleared')


class WebDataMessageDisplay(Resource):

    def __init__(self, name, storage):
        Resource.__init__(self)
        self.name = name
        self.storage = storage

    def render_GET(self, request):
        # Don't try to output the attachments over the data api.
        email_and_attachment_list = self.storage.get_for_name(self.name)
        email_list = [str(email) for email, _ in email_and_attachment_list]

        data = {'email_list': email_list,
                'address_name': self.name, }

        env = Environment(loader=PackageLoader('fakeemail', 'templates'))
        template = env.get_template('data_message_display.html')

        data_dump = {'data': json.dumps(data)}

        return template.render(data_dump).encode('utf-8')


class WebDataRootDisplay(Resource):

    def __init__(self, name, storage):
        Resource.__init__(self)
        self.storage = storage

    def internal_render(self, user_message=''):
        address_list = [[self.storage.get_count(name), name]
                        for name in self.storage.get_all_names()]
        data = {'address_list': address_list, 'user_message': user_message}

        env = Environment(loader=PackageLoader('fakeemail', 'templates'))
        template = env.get_template('data_home_page.html')

        data_dump = {'data': json.dumps(data)}

        return template.render(data_dump).encode('utf-8')

    def render_GET(self, request, user_message=None):

        return self.internal_render()

    def render_POST(self, request):
        self.storage.clear_all()
        return self.internal_render('Data Cleared')

    def getChild(self, name, request):
        return WebDataMessageDisplay(name, self.storage)


class WebMessageRouter(Resource):

    def __init__(self, storage, temp_dir):
        Resource.__init__(self)
        self.storage = storage
        self.temp_dir = temp_dir

    def getChild(self, name, request):
        if name:
            if name == 'data':
                return WebDataRootDisplay('', self.storage)
            elif name == 'file':
                return File(self.temp_dir)

            return WebMessageDisplay(name, self.storage)
        else:
            return WebMessageRootDisplay(name, self.storage)


def makeWebService(web_interface, web_port, storage):
    root = WebMessageRouter(storage)
    siteFactory = Site(root)
    webService = internet.TCPServer(web_port, siteFactory,
                                    interface=web_interface)
    return webService



from __future__ import print_function
import email
import os
import tempfile

from twisted.application import service
from twisted.python import usage
from zope.interface import implements

from smtp_server import WebMessageESMTPFactory, makeSMTPService
from web_server import WebMessageRouter, Site, makeWebService


_TEMPDIR = None
def get_tempdir():
    global _TEMPDIR
    if not _TEMPDIR:
        _TEMPDIR = tempfile.mkdtemp('fakeemail')
    return _TEMPDIR


class WebMessageStorage(object):

    messages = {}

    def addMessage(self, to, message):
        if unicode(to.dest) in self.messages:
            self.messages[unicode(to.dest)].append(self.process_message(message))
        else:
            self.messages[unicode(to.dest)] = [self.process_message(message)]

        print("Message stored for: " + unicode(to.dest))

    def process_message(self, message):
        # Unpack each message and store the attachments to a temporary
        # directory, and retain the local path where we have stored
        # them. Return the message and the list of local paths.
        msg = email.message_from_string(message)
        attachments_paths = []
        for part in msg.walk():
            content_type = part.get_content_type()
            if (part.get_content_maintype() != 'multipart') and not content_type.startswith('text/'):
                # Then we assume we have an attachment to store
                fpath = os.path.join(get_tempdir(), part.get_filename())
                with open(fpath, 'wb') as f:
                    f.write(part.get_payload(decode=True))
                attachments_paths.append(fpath)

        return msg, attachments_paths

    def get_for_name(self, name):
        if name in self.messages:
            return self.messages[name]
        return None

    def get_all_names(self):
        return self.messages.keys()

    def get_count(self, name):
        return len(self.messages[name])

    def clear_all(self):
        self.messages = {}


class Options(usage.Options):
    optParameters = [
        ["smtp_port", "s", 2025, "SMTP Server Port"],
        ["web_port", "w", 8000, "Web Server Port"],
        ["web_interface", "i", "127.0.0.1", "Web Server Interface"],
    ]


def makeService(config):

    storage = WebMessageStorage()

    s = service.MultiService()

    smtp_port = int(config.get('smtp_port'))
    web_port = int(config.get('web_port'))
    web_interface = config.get('web_interface')

    smtpService = makeSMTPService(smtp_port, storage)
    smtpService.setServiceParent(s)

    webService = makeWebService(web_interface, web_port, storage)
    webService.setServiceParent(s)

    return s


def start(config=None):

    if not config:
        import argparse
        parser = argparse.ArgumentParser()
        parser.add_argument("smtp_port", help="SMTP Port", type=int)
        parser.add_argument("web_port", help="Web Port", type=int)
        parser.add_argument("web_interface",
                            help="Listening interface for web")
        args = parser.parse_args()

        config = {
            'smtp_port': args.smtp_port,
            'web_port': args.web_port,
            'web_interface': args.web_interface
        }

    from twisted.internet import reactor
    storage = WebMessageStorage()
    temp_dir = get_tempdir()
    smtp_factory = WebMessageESMTPFactory(storage)
    root = WebMessageRouter(storage, temp_dir)

    reactor.listenTCP(config['smtp_port'], smtp_factory)
    reactor.listenTCP(config['web_port'], Site(root),
                      interface=config['web_interface'])

    reactor.run()



from setuptools import setup

setup(
    name='fakeemail',
    version='0.10.1',
    author='Tom Wardill',
    author_email='tom@howrandom.net',
    description='A fake Email Server with a Web Front End',
    url='https://github.com/tomwardill/FakeEmail',
    zip_safe=True,
    packages=['fakeemail', 'twisted.plugins'],
    install_requires=[
        'Twisted', 'jinja2',
    ],
    package_data={
        'twisted': ['plugins/fakeemail.py'],
        'fakeemail': ['templates/*']
        },
    entry_points={
        'console_scripts': [
            'fakeemail=fakeemail.server:start'
        ]
    }
)



from zope.interface import implements
from twisted.mail import smtp
from twisted.internet import defer, protocol
from twisted.application import internet, service


class WebMessageDelivery:
    implements(smtp.IMessageDelivery)

    def __init__(self, storage):
        self.storage = storage

    def receivedHeader(self, helo, origin, recipients):
        return "Recieved: MessageDelivery"

    def validateFrom(self, helo, origin):
        # take any from address
        return origin

    def validateTo(self, user):
        # take any to
        return lambda: WebMessage(self.storage, user)


class WebMessage:
    implements(smtp.IMessage)

    def __init__(self, storage, user):
        self.lines = []
        self.storage = storage
        self.user = user

    def lineReceived(self, line):
        self.lines.append(line)

    def eomReceived(self):
        message = "\n".join(self.lines)
        self.storage.addMessage(self.user, message)
        self.lines = None
        return defer.succeed(None)

    def connectionLost(self):
        # some form of error, throw it all away
        self.lines = None


class WebMessageDeliveryFactory(object):
    implements(smtp.IMessageDeliveryFactory)

    def __init__(self, storage):
        self.storage = storage

    def getMessageDelivery(self):
        return WebMessageDelivery(self.storage)


class WebMessageESMTPFactory(protocol.ServerFactory):
    protocol = smtp.ESMTP

    storage = None

    def __init__(self, storage):
        self.storage = storage

    def buildProtocol(self, addr):
        p = self.protocol()
        p.deliveryFactory = WebMessageDeliveryFactory(self.storage)
        p.factory = self
        return p


def makeSMTPService(smtp_port, storage):
    smtpServerFactory = WebMessageESMTPFactory(storage)
    smtpService = internet.TCPServer(smtp_port, smtpServerFactory)

    return smtpService
